@@ -23,18 +23,24 @@ from qtpy.QtGui import (QClipboard, QColor, QMouseEvent, QTextFormat,
 from qtpy.QtWidgets import QApplication, QMainWindow, QPlainTextEdit, QToolTip
 
 # Local imports
-from spyder.config.gui import get_font
-from spyder.config.manager import CONF
-from spyder.py3compat import PY3, to_text_string
-from spyder.widgets.calltip import CallTipWidget, ToolTipWidget
-from spyder.widgets.mixins import BaseEditMixin
+from spyder.api.fonts import SpyderFontsMixin, SpyderFontType
+from spyder.api.widgets.mixins import SpyderWidgetMixin
 from spyder.plugins.editor.api.decoration import TextDecoration, DRAW_ORDERS
 from spyder.plugins.editor.utils.decoration import TextDecorationsManager
 from spyder.plugins.editor.widgets.completion import CompletionWidget
+from spyder.plugins.completion.api import CompletionItemKind
 from spyder.plugins.outlineexplorer.api import is_cell_header, document_cells
 from spyder.utils.palette import SpyderPalette
+from spyder.widgets.calltip import CallTipWidget, ToolTipWidget
+from spyder.widgets.mixins import BaseEditMixin
 
-class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
+
+class TextEditBaseWidget(
+    QPlainTextEdit,
+    BaseEditMixin,
+    SpyderFontsMixin,
+    SpyderWidgetMixin
+):
     """Text edit base widget"""
     BRACE_MATCHING_SCOPE = ('sof', 'eof')
     focus_in = Signal()
@@ -44,15 +50,17 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     focus_changed = Signal()
     sig_insert_completion = Signal(str)
     sig_eol_chars_changed = Signal(str)
+    sig_prev_cursor = Signal()
+    sig_next_cursor = Signal()
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, class_parent=None):
         QPlainTextEdit.__init__(self, parent)
         BaseEditMixin.__init__(self)
+        SpyderWidgetMixin.__init__(self, class_parent=class_parent)
 
         self.has_cell_separators = False
         self.setAttribute(Qt.WA_DeleteOnClose)
 
-        self.extra_selections_dict = {}
         self._restore_selection_pos = None
 
         # Trailing newlines/spaces trimming
@@ -65,7 +73,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         # Code snippets
         self.code_snippets = True
 
-        self.textChanged.connect(self.changed)
         self.cursorPositionChanged.connect(self.cursor_position_changed)
 
         self.indent_chars = " "*4
@@ -86,7 +93,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.setup_completion()
 
         self.calltip_widget = CallTipWidget(self, hide_timer_on=False)
-        self.tooltip_widget = ToolTipWidget(self, as_tooltip=True)
+        self.tooltip_widget = ToolTipWidget(self)
 
         self.highlight_current_cell_enabled = False
 
@@ -110,12 +117,17 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
         def reset_current_cell():
             self.current_cell = None
+            self.highlight_current_cell()
 
         self.textChanged.connect(reset_current_cell)
 
+        # Cache
+        self._current_cell_cursor = None
+        self._current_line_block = None
+
     def setup_completion(self):
-        size = CONF.get('main', 'completion/size')
-        font = get_font()
+        size = self.get_conf('completion/size', section='main')
+        font = self.get_font(SpyderFontType.Monospace)
         self.completion_widget.setup_appearance(size, font)
 
     def set_indent_chars(self, indent_chars):
@@ -165,7 +177,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         Returns:
             list of sourcecode.api.TextDecoration.
         """
-        return self.extra_selections_dict.get(key, [])
+        return self.decorations.get(key, [])
 
     def set_extra_selections(self, key, extra_selections):
         """Set extra selections for a key.
@@ -188,20 +200,8 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             selection.draw_order = draw_order
             selection.kind = key
 
-        self.clear_extra_selections(key)
-        self.extra_selections_dict[key] = extra_selections
-
-    def update_extra_selections(self):
-        """Add extra selections to DecorationsManager.
-
-        TODO: This method could be remove it and decorations could be
-        added/removed in set_extra_selections/clear_extra_selections.
-        """
-        extra_selections = []
-
-        for key, extra in list(self.extra_selections_dict.items()):
-            extra_selections.extend(extra)
-        self.decorations.add(extra_selections)
+        self.decorations.add_key(key, extra_selections)
+        self.update()
 
     def clear_extra_selections(self, key):
         """Remove decorations added through set_extra_selections.
@@ -209,15 +209,9 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         Args:
             key (str) name of the extra selections group.
         """
-        for decoration in self.extra_selections_dict.get(key, []):
-            self.decorations.remove(decoration)
-        self.extra_selections_dict[key] = []
+        self.decorations.remove_key(key)
         self.update()
 
-    def changed(self):
-        """Emit changed signal"""
-        self.modificationChanged.emit(self.document().isModified())
-
     def get_visible_block_numbers(self):
         """Get the first and last visible block numbers."""
         first = self.firstVisibleBlock().blockNumber()
@@ -246,16 +240,21 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     # ------Highlight current line
     def highlight_current_line(self):
         """Highlight current line"""
-        selection = TextDecoration(self.textCursor())
+        cursor = self.textCursor()
+        block = cursor.block()
+        if self._current_line_block == block:
+            return
+        self._current_line_block = block
+        selection = TextDecoration(cursor)
         selection.format.setProperty(QTextFormat.FullWidthSelection,
                                      to_qvariant(True))
         selection.format.setBackground(self.currentline_color)
         selection.cursor.clearSelection()
         self.set_extra_selections('current_line', [selection])
-        self.update_extra_selections()
 
     def unhighlight_current_line(self):
         """Unhighlight current line"""
+        self._current_line_block = None
         self.clear_extra_selections('current_line')
 
     # ------Highlight current cell
@@ -263,8 +262,22 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         """Highlight current cell"""
         if (not self.has_cell_separators or
                 not self.highlight_current_cell_enabled):
+            self._current_cell_cursor = None
             return
         cursor, whole_file_selected = self.select_current_cell()
+
+        def same_selection(c1, c2):
+            if c1 is None or c2 is None:
+                return False
+            return (
+                c1.selectionStart() == c2.selectionStart() and
+                c1.selectionEnd() == c2.selectionEnd()
+            )
+
+        if same_selection(self._current_cell_cursor, cursor):
+            # Already correct
+            return
+        self._current_cell_cursor = cursor
         selection = TextDecoration(cursor)
         selection.format.setProperty(QTextFormat.FullWidthSelection,
                                      to_qvariant(True))
@@ -274,10 +287,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             self.clear_extra_selections('current_cell')
         else:
             self.set_extra_selections('current_cell', [selection])
-            self.update_extra_selections()
 
     def unhighlight_current_cell(self):
         """Unhighlight current cell"""
+        self._current_cell_cursor = None
         self.clear_extra_selections('current_cell')
 
     def in_comment(self, cursor=None, position=None):
@@ -408,7 +421,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                                           QTextCursor.KeepAnchor)
             extra_selections.append(selection)
         self.set_extra_selections('brace_matching', extra_selections)
-        self.update_extra_selections()
 
     def cursor_position_changed(self):
         """Handle brace matching."""
@@ -425,7 +437,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             return
         cursor.movePosition(QTextCursor.PreviousCharacter,
                             QTextCursor.KeepAnchor)
-        text = to_text_string(cursor.selectedText())
+        text = str(cursor.selectedText())
         if text in (')', ']', '}'):
             forward = False
         elif text in ('(', '[', '{'):
@@ -467,6 +479,9 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         """
         if self.get_selected_text():
             QApplication.clipboard().setText(self.get_selected_text())
+        else:
+            cursor = self.select_current_line_and_sep(set_cursor=False)
+            QApplication.clipboard().setText(self.get_selected_text(cursor))
 
     def toPlainText(self):
         """
@@ -478,12 +493,11 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         # corruptions when saving files with certain combinations
         # of unicode chars on them (like the one attached on
         # spyder-ide/spyder#1546).
-        if os.name == 'nt' and PY3:
+        if os.name == 'nt':
             text = self.get_text('sof', 'eof')
             return text.replace('\u2028', '\n').replace('\u2029', '\n')\
                        .replace('\u0085', '\n')
-        else:
-            return super(TextEditBaseWidget, self).toPlainText()
+        return super().toPlainText()
 
     def keyPressEvent(self, event):
         key = event.key()
@@ -494,7 +508,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         if (ctrl or meta) and key == Qt.Key_C:
             self.copy()
         else:
-            super(TextEditBaseWidget, self).keyPressEvent(event)
+            super().keyPressEvent(event)
 
     # ------Text: get, set, ...
     def get_cell_list(self):
@@ -503,13 +517,17 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         return []
 
     def get_selection_as_executable_code(self, cursor=None):
-        """Return selected text as a processed text,
-        to be executable in a Python/IPython interpreter"""
+        """
+        Get selected text in a way that allows other plugins to execute it.
+        """
         ls = self.get_line_separator()
 
         _indent = lambda line: len(line)-len(line.lstrip())
 
         line_from, line_to = self.get_selection_bounds(cursor)
+        line_col_from, line_col_to = self.get_selection_start_end(cursor)
+        line_from_off, line_to_off = self.get_selection_offsets(cursor)
+
         text = self.get_selected_text(cursor)
         if not text:
             return
@@ -518,7 +536,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         if len(lines) > 1:
             # Multiline selection -> eventually fixing indentation
             original_indent = _indent(self.get_text_line(line_from))
-            text = (" "*(original_indent-_indent(lines[0])))+text
+            text = (" " * (original_indent - _indent(lines[0]))) + text
 
         # If there is a common indent to all lines, find it.
         # Moving from bottom line to top line ensures that blank
@@ -527,7 +545,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         min_indent = 999
         current_indent = 0
         lines = text.split(ls)
-        for i in range(len(lines)-1, -1, -1):
+        for i in range(len(lines) - 1, -1, -1):
             line = lines[i]
             if line.strip():
                 current_indent = _indent(line)
@@ -548,15 +566,14 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             else:
                 break
 
-        # Add an EOL character after the last line of code so that it gets
-        # evaluated automatically by the console and any quote characters
-        # are separated from the triple quotes of runcell
-        lines.append(ls)
-
         # Add removed lines back to have correct traceback line numbers
         leading_lines_str = ls * lines_removed
 
-        return leading_lines_str + ls.join(lines)
+        return (
+            leading_lines_str + ls.join(lines),
+            (line_from_off, line_to_off),
+            (line_col_from, line_col_to)
+        )
 
     def get_cell_as_executable_code(self, cursor=None):
         """Return cell contents as executable code."""
@@ -571,10 +588,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         if not is_cell_header(block) and start > 0:
             block = self.document().findBlock(start - 1)
         # Get text
-        text = self.get_selection_as_executable_code(cursor)
+        text, off_pos, col_pos = self.get_selection_as_executable_code(cursor)
         if text is not None:
             text = ls * line_from + text
-        return text, block
+        return text, block, off_pos, col_pos
 
     def select_current_cell(self, cursor=None):
         """
@@ -630,6 +647,44 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
         return cursor, cell_full_file
 
+    def select_current_line_and_sep(self, cursor=None, set_cursor=True):
+        """
+        Selects the current line, including the correct line separator to
+        delete or copy the whole current line.
+
+        This means:
+        - If there is a next block, select the current block's newline char.
+        - Else if there is a previous block, select the previous newline char.
+        - Else select no newline char (1-line file)
+
+        Does a similar thing to `cursor.select(QTextCursor.BlockUnderCursor)`,
+        which always selects the previous newline char.
+        """
+        if cursor is None:
+            cursor = self.textCursor()
+
+        cursor.movePosition(QTextCursor.StartOfBlock, QTextCursor.MoveAnchor)
+
+        if not cursor.movePosition(QTextCursor.NextBlock,
+                                   QTextCursor.KeepAnchor):
+            # if there is no next Block, we select the previous newline
+            if cursor.movePosition(QTextCursor.PreviousBlock,
+                                   QTextCursor.MoveAnchor):
+                cursor.movePosition(QTextCursor.EndOfBlock,
+                                    QTextCursor.MoveAnchor)
+                cursor.movePosition(QTextCursor.NextBlock,
+                                    QTextCursor.KeepAnchor)
+                cursor.movePosition(QTextCursor.EndOfBlock,
+                                    QTextCursor.KeepAnchor)
+            else:
+                # if there is no previous block, we can select the current line
+                # this is the 1-line file case
+                cursor.select(QTextCursor.BlockUnderCursor)
+
+        if set_cursor:
+            self.setTextCursor(cursor)
+        return cursor
+
     def go_to_next_cell(self):
         """Go to the next cell of lines"""
         cursor = self.textCursor()
@@ -672,7 +727,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         if self._restore_selection_pos is not None:
             self.__restore_selection(*self._restore_selection_pos)
             self._restore_selection_pos = None
-        super(TextEditBaseWidget, self).paintEvent(e)
+        super().paintEvent(e)
 
     def __save_selection(self):
         """Save current cursor selection and return position bounds"""
@@ -693,13 +748,13 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         cur_pos = cursor.position()
         start_pos, end_pos = self.__save_selection()
         end_pos_orig = end_pos
-        if to_text_string(cursor.selectedText()):
+        if str(cursor.selectedText()):
             cursor.setPosition(end_pos)
             # Check if end_pos is at the start of a block: if so, starting
             # changes from the previous block
             cursor.movePosition(QTextCursor.StartOfBlock,
                                 QTextCursor.KeepAnchor)
-            if not to_text_string(cursor.selectedText()):
+            if not str(cursor.selectedText()):
                 cursor.movePosition(QTextCursor.PreviousBlock)
                 end_pos = cursor.position()
 
@@ -736,8 +791,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         cursor.insertText(text)
         cursor.endEditBlock()
 
-        self.document_did_change()
-
     def duplicate_line_down(self):
         """
         Copy current line or selected text and paste the duplicated text
@@ -752,7 +805,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         """
         self.__duplicate_line_or_selection(after_current_line=True)
 
-    def __move_line_or_selection(self, after_current_line=True):
+    def move_line_or_selection(self, after_current_line=True):
         """Move current line or selected text"""
         cursor = self.textCursor()
         cursor.beginEditBlock()
@@ -795,12 +848,12 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             return
 
         # ------ Move text
-        sel_text = to_text_string(cursor.selectedText())
+        sel_text = str(cursor.selectedText())
         cursor.removeSelectedText()
 
         if after_current_line:
             # Shift selection down
-            text = to_text_string(cursor.block().text())
+            text = str(cursor.block().text())
             sel_text = os.linesep + sel_text[0:-1]  # Move linesep at the start
             cursor.movePosition(QTextCursor.EndOfBlock)
             start_pos += len(text)+1
@@ -817,7 +870,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                 end_pos += 1
             else:
                 cursor.movePosition(QTextCursor.PreviousBlock)
-            text = to_text_string(cursor.block().text())
+            text = str(cursor.block().text())
             start_pos -= len(text)+1
             end_pos -= len(text)+1
 
@@ -827,16 +880,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.setTextCursor(cursor)
         self.__restore_selection(start_pos, end_pos)
 
-        self.document_did_change()
-
-    def move_line_up(self):
-        """Move up current line or selected text"""
-        self.__move_line_or_selection(after_current_line=False)
-
-    def move_line_down(self):
-        """Move down current line or selected text"""
-        self.__move_line_or_selection(after_current_line=True)
-
     def go_to_new_line(self):
         """Go to the end of the current line and create a new line"""
         self.stdkey_end(False, False)
@@ -855,29 +898,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                                 QTextCursor.KeepAnchor)
         self.setTextCursor(cursor)
 
-    def delete_line(self, cursor=None):
-        """Delete current line."""
-        if cursor is None:
-            cursor = self.textCursor()
-        if self.has_selected_text():
-            self.extend_selection_to_complete_lines()
-            start_pos, end_pos = cursor.selectionStart(), cursor.selectionEnd()
-            cursor.setPosition(start_pos)
-        else:
-            start_pos = end_pos = cursor.position()
-        cursor.beginEditBlock()
-        cursor.setPosition(start_pos)
-        cursor.movePosition(QTextCursor.StartOfBlock)
-        while cursor.position() <= end_pos:
-            cursor.movePosition(QTextCursor.EndOfBlock, QTextCursor.KeepAnchor)
-            if cursor.atEnd():
-                break
-            cursor.movePosition(QTextCursor.NextBlock, QTextCursor.KeepAnchor)
-        cursor.removeSelectedText()
-        cursor.endEditBlock()
-        self.ensureCursorVisible()
-        self.document_did_change()
-
     def set_selection(self, start, end):
         cursor = self.textCursor()
         cursor.setPosition(start)
@@ -910,7 +930,8 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         self.completion_widget.item_selected()
 
     def insert_completion(self, completion, completion_position):
-        """Insert a completion into the editor.
+        """
+        Insert a completion into the editor.
 
         completion_position is where the completion was generated.
 
@@ -938,25 +959,86 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
                 end = self.get_position_line_number(end_line, end_col)
             cursor.setPosition(start)
             cursor.setPosition(end, QTextCursor.KeepAnchor)
-            text = to_text_string(completion['textEdit']['newText'])
+            text = str(completion['textEdit']['newText'])
         else:
             text = completion
+            kind = None
             if isinstance(completion, dict):
                 text = completion['insertText']
-            text = to_text_string(text)
+                kind = completion['kind']
+            text = str(text)
 
-            # Get word on the left of the cursor.
-            result = self.get_current_word_and_position(completion=True)
+            # Get word to the left of the cursor.
+            result = self.get_current_word_and_position(
+                completion=True, valid_python_variable=False)
             if result is not None:
                 current_text, start_position = result
                 end_position = start_position + len(current_text)
-                # Check if the completion position is in the expected range
-                if not start_position <= completion_position <= end_position:
-                    return
-                cursor.setPosition(start_position)
-                # Remove the word under the cursor
-                cursor.setPosition(end_position,
-                                   QTextCursor.KeepAnchor)
+
+                # Remove text under cursor only if it's not an autocompletion
+                # character
+                is_auto_completion_character = False
+                if self.objectName() == 'console':
+                    if current_text == '.':
+                        is_auto_completion_character = True
+                else:
+                    if (
+                        kind != CompletionItemKind.FILE and
+                        current_text in self.auto_completion_characters
+                    ):
+                        is_auto_completion_character = True
+
+                # Adjustments for file completions
+                if kind == CompletionItemKind.FILE:
+                    special_chars = ['"', "'", '/', '\\']
+
+                    if any(
+                        [current_text.endswith(c) for c in special_chars]
+                    ):
+                        # This is necessary when completions are requested next
+                        # to special characters.
+                        start_position = end_position
+                    elif current_text.endswith('.') and len(current_text) > 1:
+                        # This inserts completions for files or directories
+                        # that start with a dot
+                        start_position = end_position - 1
+                    elif current_text == '.':
+                        # This is needed if users are asking for file
+                        # completions to the right of a dot when some of its
+                        # name is part of the completed text
+                        cursor_1 = self.textCursor()
+                        found_start = False
+
+                        # Select text backwards until we find where the file
+                        # name starts
+                        while not found_start:
+                            cursor_1.movePosition(
+                                QTextCursor.PreviousCharacter,
+                                QTextCursor.KeepAnchor,
+                            )
+
+                            selection = str(cursor_1.selectedText())
+                            if text.startswith(selection):
+                                found_start = True
+
+                        current_text = str(cursor_1.selectedText())
+                        start_position = cursor_1.selectionStart()
+                        end_position = cursor_1.selectionEnd()
+
+                if not is_auto_completion_character:
+                    # Check if the completion position is in the expected range
+                    if not (
+                        start_position <= completion_position <= end_position
+                    ):
+                        return
+                    cursor.setPosition(start_position)
+
+                    # Remove the word under the cursor
+                    cursor.setPosition(end_position, QTextCursor.KeepAnchor)
+                else:
+                    # Check if we are in the correct position
+                    if cursor.position() != completion_position:
+                        return
             else:
                 # Check if we are in the correct position
                 if cursor.position() != completion_position:
@@ -974,7 +1056,6 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             self.insert_text(text)
         else:
             self.sig_insert_completion.emit(text)
-            self.document_did_change()
 
     def is_completion_widget_visible(self):
         """Return True is completion list widget is visible"""
@@ -987,7 +1068,12 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
     def hide_completion_widget(self, focus_to_parent=True):
         """Hide completion widget and tooltip."""
-        self.completion_widget.hide(focus_to_parent=focus_to_parent)
+        # This is necessary to catch an error when creating new editor windows.
+        # Fixes spyder-ide/spyder#19109
+        try:
+            self.completion_widget.hide(focus_to_parent=focus_to_parent)
+        except RuntimeError:
+            pass
         QToolTip.hideText()
 
     # ------Standard keys
@@ -1044,10 +1130,17 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     # ----Qt Events
     def mousePressEvent(self, event):
         """Reimplement Qt method"""
+
+        # mouse buttons for forward and backward navigation
+        if event.button() == Qt.XButton1:
+            self.sig_prev_cursor.emit()
+        elif event.button() == Qt.XButton2:
+            self.sig_next_cursor.emit()
+
         if sys.platform.startswith('linux') and event.button() == Qt.MidButton:
             self.calltip_widget.hide()
             self.setFocus()
-            event = QMouseEvent(QEvent.MouseButtonPress, event.pos(),
+            event = QMouseEvent(QEvent.MouseButtonPress, event.position(),
                                 Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)
             QPlainTextEdit.mousePressEvent(self, event)
             QPlainTextEdit.mouseReleaseEvent(self, event)
@@ -1080,7 +1173,10 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     def wheelEvent(self, event):
         """Reimplemented to emit zoom in/out signals when Ctrl is pressed"""
         # This feature is disabled on MacOS, see spyder-ide/spyder#1510.
-        if sys.platform != 'darwin':
+        if (
+            sys.platform != 'darwin'
+            and not self.get_conf('disable_zoom_mouse', section='main')
+        ):
             if event.modifiers() & Qt.ControlModifier:
                 if hasattr(event, 'angleDelta'):
                     if event.angleDelta().y() < 0:
@@ -1110,8 +1206,7 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
 
     def position_widget_at_cursor(self, widget):
         # Retrieve current screen height
-        desktop = QApplication.desktop()
-        srect = desktop.availableGeometry(desktop.screenNumber(widget))
+        srect = self.screen().availableGeometry()
 
         left, top, right, bottom = (srect.left(), srect.top(),
                                     srect.right(), srect.bottom())
@@ -1125,10 +1220,12 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
         point = self.cursorRect().bottomRight()
         point = self.calculate_real_position(point)
         point = self.mapToGlobal(point)
+
         # Move to left of cursor if not enough space on right
         widget_right = point.x() + widget.width()
         if widget_right > right:
             point.setX(point.x() - widget.width())
+
         # Push to right if not enough space on left
         if point.x() < left:
             point.setX(left)
@@ -1141,6 +1238,13 @@ class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
             point = self.mapToGlobal(point)
             point.setX(x_position)
             point.setY(point.y() - widget.height())
+            delta_y = -2
+        else:
+            delta_y = 5 if sys.platform == "darwin" else 6
+
+        # Add small delta to the vertical position so that the widget is not
+        # shown too close to the text
+        point.setY(point.y() + delta_y)
 
         if ancestor is not None:
             # Useful only if we set parent to 'ancestor' in __init__
