@@ -16,6 +16,7 @@ import re
 import sys
 
 # Third party imports
+from qtpy import PYSIDE2
 from qtpy.QtCore import QEvent, QSize, Qt, QTimer, Signal, Slot
 from qtpy.QtGui import QPixmap, QTextCursor
 from qtpy.QtWidgets import (QAction, QGridLayout, QHBoxLayout, QLabel,
@@ -25,7 +26,6 @@ from qtpy.QtWidgets import (QAction, QGridLayout, QHBoxLayout, QLabel,
 # Local imports
 from spyder.api.shortcuts import SpyderShortcutsMixin
 from spyder.config.base import _
-from spyder.py3compat import to_text_string
 from spyder.utils.icon_manager import ima
 from spyder.utils.misc import regexp_error_msg
 from spyder.plugins.editor.utils.editor import TextHelper
@@ -71,7 +71,10 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
     return_pressed = Signal()
 
     def __init__(self, parent, enable_replace=False):
-        QWidget.__init__(self, parent)
+        if not PYSIDE2:
+            super().__init__(parent)
+        else:
+            QWidget.__init__(self, parent)
         SpyderShortcutsMixin.__init__(self)
         self.enable_replace = enable_replace
         self.editor = None
@@ -331,6 +334,7 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
             ("find previous", self.find_previous, parent),
             ('find text', self.show, parent),
             ('replace text', self.show_replace, parent),
+            ('replace all', self.replace_find_all, parent),
             ('hide find and replace', self.hide, self),
         )
 
@@ -372,7 +376,7 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
                     try:
                         cursor = self.editor.textCursor()
                         cursor.select(QTextCursor.WordUnderCursor)
-                        text = to_text_string(cursor.selectedText())
+                        text = str(cursor.selectedText())
                     except AttributeError:
                         # We can't do this for all widgets, e.g. WebView's
                         pass
@@ -522,7 +526,7 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
         # selection. Otherwise we show matching items.
         if multiline_replace_check and self.replace_widgets[0].isVisible():
             sel_text = self.editor.get_selected_text()
-            if len(to_text_string(sel_text).splitlines()) > 1:
+            if len(str(sel_text).splitlines()) > 1:
                 return None
         text = self.search_text.currentText()
         if len(text) == 0:
@@ -581,8 +585,8 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
         """Replace and find."""
         if self.editor is None:
             return
-        replace_text = to_text_string(self.replace_text.currentText())
-        search_text = to_text_string(self.search_text.currentText())
+        replace_text = str(self.replace_text.currentText())
+        search_text = str(self.search_text.currentText())
         re_pattern = None
         case = self.case_button.isChecked()
         re_flags = re.MULTILINE if case else re.IGNORECASE | re.MULTILINE
@@ -599,7 +603,7 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
                 return
 
         # First found
-        seltxt = to_text_string(self.editor.get_selected_text())
+        seltxt = str(self.editor.get_selected_text())
         cmptxt1 = search_text if case else search_text.lower()
         cmptxt2 = seltxt if case else seltxt.lower()
         do_replace = True
@@ -623,7 +627,7 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
                 cursor.removeSelectedText()
                 cursor.insertText(replace_text)
             else:
-                seltxt = to_text_string(cursor.selectedText())
+                seltxt = str(cursor.selectedText())
 
                 # Note: If the selection obtained from an editor spans a line
                 # break, the text will contain a Unicode U+2029 paragraph
@@ -702,8 +706,8 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
     def replace_find_selection(self, focus_replace_text=False):
         """Replace and find in the current selection"""
         if self.editor is not None:
-            replace_text = to_text_string(self.replace_text.currentText())
-            search_text = to_text_string(self.search_text.currentText())
+            replace_text = str(self.replace_text.currentText())
+            search_text = str(self.search_text.currentText())
             case = self.case_button.isChecked()
             word = self.words_button.isChecked()
             re_flags = re.MULTILINE if case else re.IGNORECASE | re.MULTILINE
@@ -728,7 +732,7 @@ class FindReplace(QWidget, SpyderShortcutsMixin):
                 # Do nothing with an invalid regexp
                 return
 
-            selected_text = to_text_string(self.editor.get_selected_text())
+            selected_text = str(self.editor.get_selected_text())
             replacement = re_pattern.sub(replace_text, selected_text)
             if replacement != selected_text:
                 cursor = self.editor.textCursor()
